from pydantic import BaseModel, Field
from typing import Optional


class SearchResult(BaseModel):
    title: str
    snippet: Optional[str]
    size: Optional[int]
    page_id: Optional[int] = Field(alias="pageid")


class Search(BaseModel):
    __root__: list[SearchResult]



from pydantic import BaseModel
from tghtml import TgHTML

from typing import Optional


class Redirect(BaseModel):
    pass


class Page(BaseModel):
    title: str
    redirects: list[Redirect]
    text: str

    tag_blocklist: Optional[list]

    @property
    def parsed(self):
        return str(TgHTML(self.text, self.tag_blocklist, is_wikipedia=False))



from dataclasses import dataclass

from .clients import BaseClient, MediaWiki, MediaWiki_Lurk
from .drivers import HttpxDriver
from .models import URL
from .exceptions import NotFound


@dataclass
class Wikipya:
    lang: str = "ru"
    base_url: str = "https://{lang}.wikipedia.org/w/api.php"
    prefix: str = "/w"
    client: BaseClient = MediaWiki
    is_lurk: bool = False
    img_blocklist: list = ()

    def __post_init__(self):
        self.url = URL(self.base_url, self.lang, self.prefix)

        self.client = MediaWiki_Lurk if self.is_lurk else self.client
        self.client = self.client(url=self.url, prefix=self.prefix, driver=HttpxDriver,
                                  img_blocklist=self.img_blocklist)

    def get_instance(self) -> MediaWiki:
        return self.client



from .page import Page
from .image import Image
from .images import Images
from .search import Search, SearchResult
from .opensearch import OpenSearch, OpenSearchResult
from .url import URL
from .summary import Summary


__all__ = (
    "Page",
    "Image", "Images",
    "Search", "SearchResult",
    "OpenSearch", "OpenSearchResult",
    "URL",
    "Summary"
)



from dataclasses import dataclass, field
from typing import Any


@dataclass
class OpenSearch:
    query: str
    _variants: list = field(default_factory=list, repr=False)
    descriptions: list = field(default_factory=list, repr=False)
    links: list = field(default_factory=list, repr=False)
    results: Any = field(default_factory=list)

    def __post_init__(self):
        for i, variant in enumerate(self._variants):
            link = self.links[i] if len(self.links) > 0 else None

            self.results.append(OpenSearchResult(variant, link))


@dataclass
class OpenSearchResult:
    title: str
    link: str = None



import re

from dataclasses import dataclass
from typing import Optional


@dataclass
class URL:
    base_url: str = "https://{lang}.wikipedia.org/w/api.php"
    lang: Optional[str] = "ru"
    prefix: str = "/w"

    def __str__(self):
        return self.url

    @property
    def url(self):
        return self.base_url.format(lang=self.lang)

    @property
    def image_url(self):
        return re.sub(r"(wiki|w)?/api.php", self.prefix, self.url)

    @property
    def cleaned(self):
        return re.sub(r"(wiki|w)?/api.php", "", self.url)



from .base import BaseClient
from ..models import Page, Search, SearchResult, Image, OpenSearch, Summary
from ..exceptions import NotFound


class MediaWiki(BaseClient):
    WGR_FLAG = (
        "https://upload.wikimedia.org/wikipedia/commons/thumb" +
        "/8/85/Flag_of_Belarus.svg/1000px-Flag_of_Belarus.svg.png"
    )

    WRW_FLAG = (
        "https://upload.wikimedia.org/wikipedia/commons/thumb" +
        "/5/50/Flag_of_Belarus_%281918%2C_1991%E2%80%931995%29.svg" +
        "/1000px-Flag_of_Belarus_%281918%2C_1991%E2%80%931995%29.svg.png"
    )

    LANG = None

    async def search(self, query, limit=1, prop="snippet") -> list[SearchResult]:
        res = await self.driver.get(
            list="search",
            srsearch=query,
            srlimit=limit,
            srprop=prop
        )

        results = Search.parse_obj(res.json["query"]["search"]).__root__

        if len(results) == 0:
            raise NotFound("Search can't find anything on your request")
        else:
            return results

    async def opensearch(self, query, limit=1) -> OpenSearch:
        r = await self.driver.get(
            action="opensearch",
            search=query,
            limit=limit,
        )

        results = OpenSearch(*r.json)

        if len(results.results) == 0:
            raise NotFound("OpenSearch can't find anything on your request")

        return results

    async def page(self, query, section=0, prop="text", blocklist=()) -> Page:
        if query.__class__ == str:
            params = {"page": query}
        elif query.__class__ == int:
            params = {"pageid": query}
        elif query.pageid is not None:
            params = {"pageid": query.pageid}
        else:
            params = {"page": query.title}

        res = await self.driver.get(
            action="parse",
            section=section,
            prop=prop,
            redirects="true",
            **params,
        )

        try:
            res.json["parse"]["text"] = res.json["parse"]["text"]["*"]
        except Exception:
            pass

        page = Page.parse_obj(res.json["parse"])
        page.tag_blocklist = blocklist

        return page

    async def summary(self, title) -> Summary:
        res = await self.driver.get_html(
            f"{self.driver.url.cleaned}api/rest_v1/page/summary/{title}")

        return Summary.parse_raw(res.text)

    async def image(self, titles, pithumbsize=1000, img_blocklist=()) -> Image:
        res = await self.driver.get(
            titles=titles,
            prop="pageimages",
            pilicense="any",
            pithumbsize=pithumbsize,
            piprop="thumbnail",
        )

        try:
            image = res.json["query"]["pages"][-1]
            thumb = image["thumbnail"]

            return Image(**thumb)
        except AttributeError:
            raise NotFound("Not found image")

    async def get_page_name(self, id) -> str:
        res = await self.driver.get(pageids=id)

        try:
            return res.json["query"]["pages"][-1]["title"]
        except AttributeError:
            raise NotFound(f"Not found page with this id: {id}")

    async def get_all(self, query, lurk=False,
                      blocklist=(), prefix="w", img_blocklist=(), **kwargs):
        search = await self.search(query)
        opensearch = await self.opensearch(
            query if lurk else search[0].title
        )

        result = opensearch.results[0]
        page = await self.page(result.title, blocklist=blocklist)

        try:
            image = await self.image(page.title, prefix=prefix, img_blocklist=img_blocklist)
            image = image.source

        except Exception:
            image = -1

        if image == self.WGR_FLAG:
            image = self.WRW_FLAG

        return page, image, result.link



from dataclasses import dataclass, field

from ..drivers import BaseDriver, HttpxDriver


@dataclass
class BaseClient:
    url: str
    prefix: str
    driver: BaseDriver = field(repr=False, default_factory=HttpxDriver)

    img_blocklist: list = field(default_factory=list, repr=False)

    DEFAULT_PARAMS = {
        "format": "json",
        "action": "query",
        "formatversion": 2
    }

    def __post_init__(self):
        self.driver = self.driver(self.url, params=self.DEFAULT_PARAMS)

    async def search(self, *args, **kwargs):
        raise NotImplementedError

    async def opensearch(self, *args, **kwargs):
        raise NotImplementedError

    async def page(self, *args, **kwargs):
        raise NotImplementedError

    async def _page(self, *args, **kwargs):
        raise NotImplementedError

    async def image(self, *args, **kwargs):
        raise NotImplementedError

    async def get_all(self, *args, **kwargs):
        raise NotImplementedError

    async def get_page_name(self, *args, **kwargs):
        raise NotImplementedError



from .base import BaseClient
from .mediawiki import MediaWiki
from .mediawiki_legacy import MediaWiki_Legacy
from .mediawiki_lurk import MediaWiki_Lurk


__all__ = ("BaseClient", "MediaWiki", "MediaWiki_Legacy", "MediaWiki_Lurk")



try:
    import httpx
except ImportError:
    print("Not found httpx. Try pip install httpx")

from dataclasses import dataclass, field
from typing import Optional

from .exceptions import ParseError


@dataclass
class BaseDriver:
    url: str = "example.org"
    timeout: Optional[int] = 5
    params: dict = field(default_factory=dict)

    async def get(self, url, params=(), timeout=None):
        raise NotImplementedError


@dataclass
class HttpxDriver(BaseDriver):
    def __post_init__(self):
        self._client = httpx.AsyncClient(timeout=self.timeout)

    async def get(self, url=None, timeout=None, debug=False, **params):
        res = await self._client.get(str(url or self.url), params={**self.params, **params})
        res.json = res.json()

        if not isinstance(res.json, list) and (error := res.json.get("error")):
            raise ParseError(f'{error["code"]}: {error["info"]}')

        return res

    async def get_html(self, url=None, timeout=None, debug=False, **params):
        return await self._client.get(str(url or self.url), params=params, follow_redirects=True)



from pydantic import BaseModel, Field
from typing import Optional

from .image import Image


class DesktopUrls(BaseModel):
    page: str


class ContentUrls(BaseModel):
    desktop: DesktopUrls


class Summary(BaseModel):
    title: str
    page_id: int = Field(alias="pageid")

    thumbnail: Optional[Image]
    original_image: Optional[Image] = Field(alias="originalimage")

    description: Optional[str]

    extract: str
    extract_html: str

    content_urls: ContentUrls



from .mediawiki import MediaWiki
from ..models import Images, Image
from ..exceptions import NotFound

from bs4 import BeautifulSoup


class MediaWiki_Legacy(MediaWiki):
    async def image(self, titles, pithumbsize=1000, img_blocklist=()) -> Image:
        r = await self.driver.get(
            action="parse",
            page=titles,
            prop="images",
            redirects="true",
            section=0
        )

        _images = Images.parse_obj(r.json["parse"]).images
        images = []

        if len(_images) == 0:
            raise NotFound("Not found images")

        for image in _images:
            if image in img_blocklist:
                continue

            try:
                images.append(await self.get_image(image))
                return images[0]
            except Exception as e:
                print(e)

    async def get_image(self, name) -> Image:
        r = await self.driver.get_html(f"{self.url.image_url}/File:{name}")
        soup = BeautifulSoup(r.text, 'lxml')

        file = soup.find("div", id="file")
        thumbnails = soup.find_all("img", {"class": "pi-image-thumbnail"})
        thumbs = soup.find_all("figure", {"class": "thumb"})
        navigation = soup.find_all("nav", {"class": "pi-navigation"})

        if file is not None:
            image = file.a.img
            image["src"] = f"https:{image['src']}"

        elif len(thumbnails) != 0:
            image = thumbnails[0]
            image["src"] = image["srcset"].split()[-2]

        elif len(navigation) != 0:
            image = navigation[0].a.img

        elif len(thumbs) != 0:
            image = thumbs[0].a.img

        else:
            raise NotFound("Not found image")

        return Image(
            source=image["src"],
            width=int(image["width"]), height=int(image["height"])
        )
